%DTMFDETECTOR	DTMF detection program with 8-bit arithmetic
%			DTMFDETECTOR(X, Y) accepts noisy signals with DTMF
%			digits and detects ITU-compliant DTMF digits. ITU
%			has stringent conditions for decoding DTMF digits.
%			The DTMF digits are valid only if they satisfy all
%			the ITU requirements.
%			
%			The program uses 8-bit wrap-around arithmetic. 
%			Three functions, 'mymul', 'myplus', and 'mysub'
%			implement 8-bit multiplication, addition and
%			subtraction respectively.
%			
%			Input X is noisy signal containing DTMF symbols. At
%			the start, the input signal should have a non-signal 
%			duration of atleast 40-50 ms for the power estimator 
%			to stabilize. The last valid DTMF symbol in the signal 
%			should be followed by a non-signal duration of atleast
%			15-20 ms. This is essential because ITU requires a
%			pause of at least 10 ms after a valid DTMF symbol.
%			
%			The input signal X needs is converted internally from
%			floating point to 8-bit fixed point signal. Input Y
%			decides the amplitude range of the converted signal.
%			The signal amplitude ranges from -Y/2 to +Y/2. Thus, 
%			Y effectively decides the attenuation suffered by the 
%			signal. Valid range for Y is 2-256. 
%			
%			The function outputs an array containing the ITU-compliant
%			DTMF symbols detected in the signal. The output symbols 
%			range from 1 to 16. The relationship of output symbols to
%			the actual DTMF symbols is shown below:
%			Top row represents output symbol.
%			Bottom row represents actual DTMF symbol.
%			
%			1  2  3  4  5  6  7  8  9  10  11  12  13  14  15  16
%			------------------------------------------------------
%			1  2  3  A  4  5  6  B  7   8   9   C   *   0   #   D
%
%			If no ITU-compliant DTMF symbols are detected, the function
%			outputs -1.
			
% Copyright (c) 1998-1999 The University of Texas
% All rights reserved.
% See the file COPYRIGHT for copyright notice, limitation
% of liability, and disclaimer of warranty provisions.

function key = dtmfdetector(signal,max_amplitude);	% Function declaration

%{----------------- CONSTANTS ---------------------}

% 300 Hz dial tone suppressor coefficient
notch300_mu1 = 114;				% Cos(2*pi*300/4000)*1/2

% Adaptive notch filter coefficients
Cos_1633Hz_mu1 = -107;			% Cos(2*pi*1633/4000)*1/2
Cos_941Hz_mu1 = 12;				% Cos(2*pi*941/4000)*1/2

% Low pass filter coefficients
lpf_mu1 = 224;					% 0.875
lpf_mu2 = 32;					% (1-0.875)

% Signal power estimator coefficients
Spower_mu1 = 232;				% 0.95
Spower_mu2 = 40;				% (1-0.95)

% Cosine value computation coefficients
mu_866 = 222;					% 0.866
mu_866_squared = 95;			% 0.866*0.866/2

% Decision logic constants
N5ms = 10;						% Samples corresponding to 5ms at 2kHz
N10ms = 20;						% Samples corresponding to 10ms at 2 kHz
N23ms = 30;						% Samples corresponding to 23ms at 2 kHz
								% (ideally N23ms should be 46. However, the detector
								% latches onto a valid DTMF symbol with a fixed delay {usually
								% 8 ms}. This delay is subtracted from 46 to calculate the
								% value of N23ms)

% High frequency check and cosine value computation constants
F1633Hz_fraction_low = 94;		% Fractional part of number of samples between two zero
								% crossings for lower end of 1633 Hz tolerance band (1576 Hz)
F1633Hz_fraction_high = 140;	% Fractional part of number of samples between two zero
								% crossings for higher end of 1633 Hz tolerance band (1690 Hz)
F1633Hz_slope = 73;				% Slope of cosine function in 1633 Hz tolerance band
F1633Hz_intercept = -100;		% Cosine of lower end of 1633 Hz tolerance band (Cos(2*pi*1576/4000)*1/2)


F1477Hz_fraction_low = 159;		% Fractional part of number of samples between two zero
								% crossings for lower end of 1477 Hz tolerance band (1425 Hz)
F1477Hz_fraction_high = 208;	% Fractional part of number of samples between two zero
								% crossings for higher end of 1477 Hz tolerance band (1528 Hz)
F1477Hz_slope = 80;				% Slope of cosine function in 1477 Hz tolerance band
F1477Hz_intercept = -78;		% Cosine of lower end of 1477 Hz tolerance band (Cos(2*pi*1425/4000)*1/2)


F1336Hz_fraction_low = 210;		% Fractional part of number of samples between two zero
								% crossings for lower end of 1336 Hz tolerance band (1289 Hz)
F1336Hz_fraction_high = 35;		% Fractional part of number of samples between two zero
								% crossings for higher end of 1336 Hz tolerance band (1382 Hz) 
F1336Hz_slope = 77;				% Slope of cosine function in 1336 Hz tolerance band
F1336Hz_intercept = -55;		% Cosine of lower end of 1336 Hz tolerance band (Cos(2*pi*1289/4000)*1/2)


F1209Hz_fraction_low = 51;		% Fractional part of number of samples between two zero
								% crossings for lower end of 1209 Hz tolerance band (1167 Hz)
F1209Hz_fraction_high = 110;	% Fractional part of number of samples between two zero
								% crossings for higher end of 1209 Hz tolerance band (1251 Hz)
F1209Hz_slope = 69;				% Slope of cosine function in 1209 Hz tolerance band
F1209Hz_intercept = -33;		% Cosine of lower end of 1209 Hz tolerance band (Cos(2*pi*1167/4000)*1/2)

% Low frequency check and cosine value computation constants
F941Hz_fraction_low = 32;		% Fractional part of number of samples between two zero
								% crossings for lower end of 941 Hz tolerance band (908 Hz)
F941Hz_fraction_high = 107;		% Fractional part of number of samples between two zero
								% crossings for higher end of 941 Hz tolerance band (974 Hz)
F941Hz_slope = 44;				% Slope of cosine function in 941 Hz tolerance band
F941Hz_intercept = 19;			% Cosine of lower end of 941 Hz tolerance band (Cos(2*pi*908/4000)*1/2) 


F852Hz_fraction_low = 139;		% Fractional part of number of samples between two zero
								% crossings for lower end of 852 Hz tolerance band (822 Hz)
F852Hz_fraction_high = 224;		% Fractional part of number of samples between two zero
								% crossings for higher end of 852 Hz tolerance band (882 Hz)
F852Hz_slope = 35;				% Slope of cosine function in 852Hz tolerance band 
F852Hz_intercept = 36;			% Cosine of lower end of 852 Hz tolerance band (Cos(2*pi*822/4000)*1/2)


F770Hz_fraction_low = 8;		% Fractional part of number of samples between two zero
								% crossings for lower end of 770 Hz tolerance band (743 Hz)
F770Hz_fraction_high = 103;		% Fractional part of number of samples between two zero
								% crossings for higher end of 770 Hz tolerance band (797 Hz)
F770Hz_slope = 27;				% Slope of cosine function in 770 Hz tolerance band
F770Hz_intercept = 51;			% Cosine of lower end of 770 Hz tolerance band (Cos(2*pi*743/4000)*1/2)


F697Hz_fraction_low = 142;		% Fractional part of number of samples between two zero
								% crossings for lower end of 697 Hz tolerance band (673 Hz)
F697Hz_fraction_high = 247;		% Fractional part of number of samples between two zero
								% crossings for higher end of 697 Hz tolerance band (721 Hz)
F697Hz_slope = 21;				% Slope of cosine function in 697 Hz tolerance band
F697Hz_intercept = 63;			% Cosine of higher end of 697 Hz tolerance band (Cos(2*pi*673/4000)*1/2)

%{----------------- VARIABLES ---------------------}

key = -1;							% Initialize the output variable

% Signal samples
sample = zeros(1,2);				% Anti-aliased signal samples
decimated_signal = zeros(1,3);		% Decimated signal samples
notch300 = zeros(1,3);				% 300 Hz notched signal samples
notchhf = zeros(1,5);				% High frequency notched signal	samples
notchdc_hf = zeros(1,3);			% DC notched signal samples
notchlf = zeros(1,5);				% Low frequency notched signal samples
notchdc_lf = zeros(1,3);			% DC notched signal samples
deltahf = zeros(1,2);				% Distance between sample and zero crossing	
deltalf = zeros(1,2);				% Distance between sample and zero crossing

% Adaptive notch filter coefficients
cos_hfreq_mu1 = Cos_1633Hz_mu1;		% High frequency coefficient
cos_lfreq_mu1 = Cos_941Hz_mu1;		% Low frequency coefficient

% Parameters to simulate microcontroller implementation
divisor = 1;						% Divisor in a division
quotient = 0;						% Quotient part of division
carry = 0;							% Carry bit 
tmp1 = 0; tmp2 = 0; tmp3 = 0;		% Temporary registers

% Power estimation and signal detection parameters
i_p = 0;							% Positive signal sample
spower = 0;							% Signal power
Noise_floor = 100;					% Adaptive noise floor
spower_counter = 80;				% Signal counter
npower_counter = 20;				% Noise counter
min_spower = 255;					% Min power in a signal burst
max_spower = 0;						% Max power when no signal present
Ncounts = 25;						% Noise counter for exponential decay of noise floor
Scounts = 80;						% Signal counter for exponential rise of noise floor

% Decision logic parameters
row = 17;							% Row of detected digit
column = 17;						% Column of detected digit
TC = 40;							% Counts non-signal duration
ST = 0;								% Counts signal duration
PS = 19;							% Detected digit in previous iteration
VS = 20;							% Previously detected valid digit 
CT = 0;								% Counter
index_count = 1;					% Array index

% High frequency parameters
period_integer_hfcycle = 0; period_fraction_hfcycle = 0; % Period integer and fraction for 1 cycle
period_integer_hfraw = 0; period_fraction_hfraw = 0;	 % Raw period integer and fraction
avg_period_fraction_hf = 0; 							 % Averaged period fraction
period_integer_hf = 0; period_fraction_hf = 0;			 % Low pass filtered period estimates

% low frequency parameters
period_integer_lfcycle = 0; period_fraction_lfcycle = 0; % Period integer and fraction for 1 cycle
period_integer_lfraw = 0; period_fraction_lfraw = 0;	 % Raw period integer and fraction
avg_period_fraction_lf = 0; 							 % Averaged period fraction
period_integer_lf = 0; period_fraction_lf = 0;			 % Low pass filtered period estimate							

%{------------- CONVERT SIGNAL TO FIXED POINT ------}

signal = signal./max(abs(signal));						 % Limit signal amplitude between -1 and 1
signal = round(signal.*(max_amplitude -(1+signal))/2);	 % Convert to integer between -128 to +127
len = length(signal);									 % Length of signal

%{----------- DETECTION STARTS HERE ----------------}
for n=1:len												 % Loop till the end of signal

%{------------- AUTOMATIC GAIN CONTROL -------------}

	% If -6 < signal strength < +6, multiply signal strength by 2
	if ( (signal(n) >=0 & signal(n) < 16) | (signal(n) <0 & signal(n) >=-16) ) 		
		signal(n) = 2*signal(n);
	end;

	% If -8 < signal strength < +8, multiply signal strength by 2
	if ( (signal(n) >=0 & signal(n) < 8) | (signal(n) <0 & signal(n) >= -8) ) 		
		signal(n) = 2*signal(n);
	end;
		
%{------------ ANTI-ALIASING FILTER ----------------}
			
	sample(2) = sample(1);
	sample(1) = floor(signal(n)/2)+floor(sample(2)/2);	 % 2-tap moving average filter

%{---------------- DECIMATE SIGNAL -----------------}

	if( rem(n,2)==0)									 % Decimate by 2
		decimated_signal(2:3) = decimated_signal(1:2);	 
		decimated_signal(1) = sample(1);

%{------------ 300 Hz NOTCH FILTER -----------------}

		notch300(2:3) = notch300(1:2);					 % FIR notch filter
		a = mymul(notch300_mu1,decimated_signal(2));	 % Transfer function is
		a = 2*a;										 % H(z) = 1 - 2*Cos(2*pi*300/4000) + 1
		d = mysub(floor(decimated_signal(1)/2),a);
		e = myplus(d,floor(decimated_signal(3)/2));
		notch300(1) = e;
			
%{----- FULLY ADAPTIVE HIGH FREQUENCY NOTCH FILTER -----}

		notchhf(2:5) = notchhf(1:4);					 % FIR notch filter
		a = mymul(cos_hfreq_mu1,notch300(2));			 % Transfer function is 
		a = 2*a;										 % H(z) = 1 - 2*Cos(2*pi*high_freq/4000) + 1 
		d = mysub(floor(notch300(1)/2),a);
		e = myplus(d,floor(notch300(3)/2));
		notchhf(1) = e;

%{------------ DC NOTCH FILTER ------------------------}

		notchdc_hf(2:3) = notchdc_hf(1:2);	
		if(row < 10)									 % For frequencies 697 Hz and 770 Hz,  
			tmp1 = mysub(floor(notchhf(1)/4),floor(notchhf(5)/4)); % use 5-tap DC notch filter
			tmp2 = mysub(floor(notchhf(2)/4),floor(notchhf(4)/4));
			notchdc_hf(1) = myplus(tmp1,tmp2);
		else											 % For frequencies 852 Hz and 941 Hz,
			notchdc_hf(1) = mysub(floor(notchhf(1)/2),floor(notchhf(3)/2)); % use 3-tap DC 
		end;											 % notch filter

%{----- FULLY ADAPTIVE LOW FREQUENCY NOTCH FILTER -----}

		notchlf(2:5) = notchlf(1:4);					 % FIR notch filter
		a = mymul(cos_lfreq_mu1,notch300(2));			 % Transfer function is 
		a = 2*a;										 % H(z) = 1 - Cos(2*pi*low_freq/4000) + 1
		d = mysub(floor(notch300(1)/2),a);
		e = myplus(d,floor(notch300(3)/2));
		notchlf(1) = e;
	
%{------------ DC NOTCH FILTER ------------------------}

		notchdc_lf(2:3) = notchdc_lf(1:2);				 % 5-tap DC notch filter
		tmp1 = mysub(floor(notchlf(1)/4),floor(notchlf(5)/4));
		tmp2 = mysub(floor(notchlf(4)/4),floor(notchlf(2)/4));
		notchdc_lf(1) = myplus(tmp1,tmp2);

%{------------ HIGH FREQUENCY DETECTION ------------}		

		% Since bitwise operations on variables are difficult in Matlab, the divide is 
		% implemented as a normal divide and result is then converted to an 8-bit
		% number. On a microcontroller or a 8-bit processor, the divide can be implemented
		% as a series of shifts and subtracts. The 'divide.c' file shows how to implement
		% such a division.

 		if(notchdc_lf(2)<0 & notchdc_lf(1)>=0)			% Is it a positive going zero crossing?
			divisor = notchdc_lf(1)-notchdc_lf(2);		% YES: find the divisor,
			quotient = notchdc_lf(1)/divisor;			% divide positive sample, and interpolate
														% the distance from zero crossing (ZC)
			deltahf(1) = floor(quotient*256);			% Convert distance to 8-bit number


			if(deltahf(2)<deltahf(1))					% IF current distance < distance for
														% previous cycle ...
				period_fraction_hfcycle = 256 - (deltahf(1) - deltahf(2)); % Calculate fractional
														% part of number of samples between 2 zero
														% crossings for current cycle
				period_integer_hfcycle = period_integer_hfcycle - 1; % Calculate integer part of
														% number of samples between 2 zero crossings
														% for current cycle
			else										% ELSE
				period_fraction_hfcycle = deltahf(2) - deltahf(1); % Calculate only the fractional
														% part of number of samples between 2 zero
														% crossings for current cycle
			end;


			period_integer_hfcycle = period_integer_hfcycle + 1; % Increment integer part of ZC count

			% Low pass filter the fractional and integer parts using an IIR filter
			% with transfer function, H(z) = 0.5/(1 - 0.5*z^-1)
			% This filtering is implemented in a manner in which it will be
			% implemented on a processor using shifts
			
			tmp1 = period_fraction_hfcycle + period_fraction_hfraw; % Add the current and previous
																	% fractional parts
			tmp2 = rem(tmp1,256);						% Extract the lower 8 bits of addition
			carry = floor(tmp1/256);					% Extract the carry bit (9th bit)
			tmp3 = period_integer_hfcycle + period_integer_hfraw + carry; % Add the current and
														% previous integer parts with carry
			
			% period_integer_hfraw = period_integer_hfraw >> 1
			% period_fraction_hfraw = period_fraction_hfraw >> 1
			
			period_integer_hfraw = floor(tmp3/2);		% Divide integer part and convert to 8 bits
			carry = rem(tmp3,2);						% Extract the carry bit
			period_fraction_hfraw = carry*128 + floor(tmp2/2); % Stick the carry bit into MSB position
														% Divide fractional part by 2
			
			% Use additional filtering for the fractional part
		
			avg_period_fraction_hf = floor((avg_period_fraction_hf+period_fraction_hfraw)/2);
		

			period_fraction_hfcycle = 0;				% Reset the fractional and integer parts
			period_integer_hfcycle = 0;					% for next cycle
			deltahf(2) = deltahf(1);					% Store current distance from ZC
		else											
			period_integer_hfcycle = period_integer_hfcycle + 1; % Increment integer part of the
		end;											% ZC count
		
%{------------ LOW FREQUENCY DETECTION ------------}	

		% Since bitwise operations on variables are difficult in Matlab, the divide is 
		% implemented as a normal divide and result is then converted to an 8-bit
		% number. On a microcontroller or a 8-bit processor, the divide can be implemented
		% as a series of shifts and subtracts. The 'divide.c' file shows how to implement
		% such a division.

 		if(notchdc_hf(2)<0 & notchdc_hf(1)>=0)			% Is it a positive going zero crossing?
			divisor = notchdc_hf(1)-notchdc_hf(2);		% YES: find the divisor,
			quotient = notchdc_hf(1)/divisor;			% divide positive sample, and interpolate
														% the distance from zero crossing (ZC)
			deltalf(1) = floor(quotient*256);			% Convert distance to 8-bit number


			if(deltalf(2)<deltalf(1))					% IF current distance < distance for
														% previous cycle ...
				period_fraction_lfcycle = 256 - (deltalf(1) - deltalf(2));% Calculate fractional
														% part of number of samples between 2 zero
														% crossings for current cycle
				period_integer_lfcycle = period_integer_lfcycle - 1;% Calculate integer part of
														% number of samples between 2 zero crossings
														% for current cycle
			else										% ELSE
				period_fraction_lfcycle = deltalf(2) - deltalf(1);% Calculate only the fractional
														% part of number of samples between 2 zero
														% crossings for current cycle
			end;

			period_integer_lfcycle = period_integer_lfcycle + 1; % Increment integer part of ZC count

			% Low pass filter the fractional and integer parts using an IIR filter
			% with transfer function, H(z) = 0.5/(1 - 0.5*z^-1)
			% This filtering is implemented in a manner in which it will be
			% implemented on a processor using shifts
	
			tmp1 = period_fraction_lfcycle + period_fraction_lfraw; % Add the current and previous
																	% fractional parts
			tmp2 = rem(tmp1,256);						% Extract the lower 8 bits of addition
			carry = floor(tmp1/256);					% Extract the carry bit (9th bit)
			tmp3 = period_integer_lfcycle + period_integer_lfraw + carry; % Add the current and
														% previous integer parts with carry

			% period_integer_hfraw = period_integer_hfraw >> 1
			% period_fraction_hfraw = period_fraction_hfraw >> 1
			
			period_integer_lfraw = floor(tmp3/2);		% Divide integer part and convert to 8 bits
			carry = rem(tmp3,2);						% Extract the carry bit
			period_fraction_lfraw = carry*128 + floor(tmp2/2); % Stick the carry bit into MSB position
														% Divide fractional part by 2

			% Use additional filtering for the fractional part
		
			avg_period_fraction_lf = floor((avg_period_fraction_lf+period_fraction_lfraw)/2);
	
			period_fraction_lfcycle = 0;				% Reset the fractional and integer parts
			period_integer_lfcycle = 0;					% for next cycle
			deltalf(2) = deltalf(1);					% Store current distance from ZC
		else
			period_integer_lfcycle = period_integer_lfcycle + 1; % Increment integer part of the
		end;											% ZC count
		
%{--------------- HIGH FREQUENCY ESTIMATE FILTER ----------------}

		% Low pass filter the zero crossings estimate of high frequency signal
		% Transfer function is F(n) = alpha*F(n-1) + (1-alpha)*F(n)
		% alpha = 0.875.

		tmp1 = mymul(period_fraction_hf,lpf_mu1);		% alpha*fraction estimate over all cycles
		tmp2 = mymul(avg_period_fraction_hf,lpf_mu2);	% (1-alpha)*averaged fraction estimate over 
														% previous cycle
		tmp3 = mymul(period_integer_hf,lpf_mu1);		% alpha*integer estimate over all cycles
		tmp4 = mymul(period_integer_hfraw,lpf_mu2);		% (1-alpha)*averaged integer estimate over
														% previous cycle
		tmp1 = tmp1 + tmp2;								% Add the fractional result
		tmp2 = tmp3 + tmp4;								% Add the integer result
		tmp3 = rem(lpf_mu1*period_integer_hf,256);		% Get fractional part of multiplication of
														% alpha and integer estimate
		tmp4 = rem(lpf_mu2*period_integer_hfraw,256);	% Get fractional part of multiplication of
														% alpha and averaged integer estimate
		tmp3 = tmp3+tmp4;								% Add the above fractional parts
		carry = floor((tmp1+tmp3)/256);					% Add the fractional part obtained after low
		period_fraction_hf = rem(tmp1+tmp3,256);		% pass filtering the fraction estimate and the
														% fractional part obtained after low pass 
														% filtering integer estimate. Extract carry bit	 
		period_integer_hf = tmp2 + carry;				% Add filtered integer estimate and carry bit
				
%{--------------- LOW FREQUENCY ESTIMATE FILTER ----------------}

		% Low pass filter the zero crossings estimate of low frequency signal
		% Transfer function is F(n) = alpha*F(n-1) + (1-alpha)*F(n)
		% alpha = 0.875.

		tmp1 = mymul(period_fraction_lf,lpf_mu1);		% alpha*fraction estimate over all cycles
		tmp2 = mymul(avg_period_fraction_lf,lpf_mu2);	% (1-alpha)*averaged fraction estimate over
														% previous cycle
		tmp3 = mymul(period_integer_lf,lpf_mu1);		% alpha*integer estimate over all cycles
		tmp4 = mymul(period_integer_lfraw,lpf_mu2);		% (1-alpha)*averaged integer estimate over
														% previous cycle
		tmp1 = tmp1 + tmp2;								% Add the fractional result
		tmp2 = tmp3 + tmp4;								% Add the integer result
		tmp3 = rem(lpf_mu1*period_integer_lf,256);		% Get fractional part of multiplication of
														% alpha and integer estimate
		tmp4 = rem(lpf_mu2*period_integer_lfraw,256);	% Get fractional part of multiplication of
														% alpha and averaged integer estimate
		tmp3 = tmp3+tmp4;								% Add the above fractional parts
		carry = floor((tmp1+tmp3)/256);					% Add the fractional part obtained after low
		period_fraction_lf = rem(tmp1+tmp3,256);		% pass filtering the fraction estimate and the
														% fractional part obtained after low pass 
														% filtering integer estimate. Extract carry bit
		period_integer_lf = tmp2 + carry;				% Add filtered integer estimate and carry bit

%{--------------- HIGH FREQUENCY CHECK --------------------------}

		% Check if the zero crossings estimate for high frequency signal lies within
		% the four high frequency tolerance bands.

		if(period_integer_hf == 2)						% If integer estimate = 2, high frequency tone can
														% lie in 1633 Hz or 1477 Hz or 1336 Hz band

			if(period_fraction_hf > F1633Hz_fraction_low & period_fraction_hf < F1633Hz_fraction_high)
														% If the fractional part lies between the values
														% for 1633 Hz tolerance band, the high frequency
														% tone is within 1633 Hz +/- 1.5% band
				column = 4;								% Set the column value for decision logic
				
				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_hf - F1633Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 1633 Hz
														% tolerance band (1570 Hz)
				tmp2 = mymul(tmp1,F1633Hz_slope);		% Multiply difference by slope of 1633 Hz band
				cos_hfreq_mu1 = F1633Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 1633 Hz tolerance band (Cos(2*pi*1570/4000))

			elseif(period_fraction_hf > F1477Hz_fraction_low & period_fraction_hf < F1477Hz_fraction_high)
														% If the fractional part lies between the values
														% for 1477 Hz tolerance band, the high frequency
														% tone is within 1477 Hz +/- 1.5% band
				column = 3;								% Set the column value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_hf - F1477Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 1477 Hz
														% tolerance band (1420 Hz)
				tmp2 = mymul(tmp1, F1477Hz_slope);		% Multiply difference by slope of 1477 Hz band
				cos_hfreq_mu1 = F1477Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 1477 Hz tolerance band (Cos(2*pi*1420/4000))

			elseif(period_fraction_hf > F1336Hz_fraction_low)
														% If the fractional part is greater than the 
														% fractional value of lower end of 1336 Hz tolerance
														%  band, the high frequency tone is within
														% 1336 Hz +/- 1.5% band
				column = 2;								% Set the column value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = 255 - period_fraction_hf;		% Subtract fraction estimate from the
				tmp1 = -tmp1 - F1336Hz_fraction_high;	% fractional value for lower end of 1336 Hz
														% tolerance band (1285 Hz)	
				tmp2 = mymul(tmp1, F1336Hz_slope);		% Multiply difference by slope of 1336 Hz band
				cos_hfreq_mu1 = F1336Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 1336 Hz tolerance band (Cos(2*pi*1285/4000))

			else										% if integer estimate = 2 but fractional part does
				column = 17;							% not fall within the values for tolerance bands,  
			end;										% set column value to 17

		elseif (period_integer_hf == 3)					% If integer estimate = 3, high frequency tone can
														% lie in 1336 Hz or 1209 Hz

			if(period_fraction_hf < F1336Hz_fraction_high)
														% If the fractional part is less than the 
														% fractional value of higher end of 1336 Hz tolerance
														%  band, the high frequency tone is within
														% 1336 Hz +/- 1.5% band
				column = 2;								% Set the column value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_hf - F1336Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 1336 Hz
														% tolerance band (1285 Hz)	
				tmp2 = mymul(tmp1,F1336Hz_slope);		% Multiply difference by slope of 1336 Hz band
				cos_hfreq_mu1 = F1336Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 1336 Hz tolerance band (Cos(2*pi*1285/4000))

			elseif(period_fraction_hf > F1209Hz_fraction_low & period_fraction_hf < F1209Hz_fraction_high)
														% If the fractional part lies between the values
														% for 1209 Hz tolerance band, the high frequency
														% tone is within 1209 Hz +/- 1.5% band
				column = 1;								% Set the column value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_hf - F1209Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 1209 Hz
														% tolerance band (1165 Hz)
				tmp2 = mymul(tmp1,F1209Hz_slope);		% Multiply difference by slope of 1209 Hz band
				cos_hfreq_mu1 = F1209Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 1209 Hz tolerance band (Cos(2*pi*1165/4000))

			else										% if integer estimate = 3 but fractional part does
				column = 17;							% not fall within the values for tolerance bands,
			end;										% set column value to 17

		else											% If the integer estimate is not equal to 2 or 3,
				column = 17;							% the frequency estimate does not lie in any of the
		end;											% tolerance bands. Set column value to 17.

%{--------------- LOW FREQUENCY CHECK --------------------------}

		% Check if the zero crossings estimate for low frequency signal lies within
		% the four low frequency tolerance bands.

		if(period_integer_lf == 4)						% If integer estimate = 4, low frequency tone can
														% lie in 941 Hz or 852 Hz band

			if(period_fraction_lf > F941Hz_fraction_low & period_fraction_lf < F941Hz_fraction_high)
														% If the fractional part lies between the values
														% for 941 Hz tolerance band, the low frequency
														% tone is within 941 Hz +/- 1.5% band
				row = 12;								% Set the row value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_lf - F941Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 941 Hz
														% tolerance band (905 Hz)
				tmp2 = mymul(tmp1,F941Hz_slope);		% Multiply difference by slope of 941 Hz band
				cos_lfreq_mu1 = F941Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 941 Hz tolerance band (Cos(2*pi*905/4000))

			elseif(period_fraction_lf > F852Hz_fraction_low & period_fraction_lf < F852Hz_fraction_high)
														% If the fractional part lies between the values
														% for 852 Hz tolerance band, the low frequency
														% tone is within 852 Hz +/- 1.5% band
				row = 8;								% Set the row value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_lf - F852Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 852 Hz
														% tolerance band (820 Hz)
				tmp2 = mymul(tmp1, F852Hz_slope);		% Multiply difference by slope of 852 Hz band
				cos_lfreq_mu1 = F852Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 852 Hz tolerance band (Cos(2*pi*820/4000))

			else										% if integer estimate = 4 but fractional part does
				row = 17;								% not fall within the values for tolerance bands,
			end;										% set row value to 17

		elseif (period_integer_lf == 5)					% If integer estimate = 4, low frequency tone can
														% lie in 770 Hz or 697 Hz band

			if(period_fraction_lf > F770Hz_fraction_low & period_fraction_lf < F770Hz_fraction_high)
														% If the fractional part lies between the values
														% for 770 Hz tolerance band, the low frequency
														% tone is within 770 Hz +/- 1.5% band
				row = 4;								% Set the row value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_lf - F770Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 770 Hz
														% tolerance band (740 Hz)
				tmp2 = mymul(tmp1,F770Hz_slope);		% Multiply difference by slope of 770 Hz band
				cos_lfreq_mu1 = F770Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 770 Hz tolerance band (Cos(2*pi*720/4000))

			elseif(period_fraction_lf > F697Hz_fraction_low & period_fraction_lf < F697Hz_fraction_high)
														% If the fractional part lies between the values
														% for 697 Hz tolerance band, the low frequency
														% tone is within 697 Hz +/- 1.5% band
				row = 0;								% Set the row value for decision logic

				% Use linear interpolation to calculate the cosine value
				tmp1 = period_fraction_lf - F697Hz_fraction_high; % Subtract fraction estimate from the
														% fractional value for lower end of 697 Hz
														% tolerance band (670 Hz)
				tmp2 = mymul(tmp1,F697Hz_slope);		% Multiply difference by slope of 697 Hz band
				cos_lfreq_mu1 = F697Hz_intercept + tmp2; % Add the result to cosine value for lower end 
														% of 697 Hz tolerance band (Cos(2*pi*670/4000))

			else										% if integer estimate = 5 but fractional part does
				row = 17;								% not fall within the values for tolerance bands,
			end;										% set row value to 17
			
		else											% If the integer estimate is not equal to 4 or 5,
			row = 17;									% the frequency estimate does not lie in any of the
		end;											% tolerance bands. Set column value to 17.

		NS = row + column;								% Update the new digit
		
%{------------- POWER ESTIMATOR AND SIGNAL DETECTOR ---------------}

		i_p = decimated_signal(1);						% Get the current sample
		if (i_p < 0)									% Get the absolute value 
			i_p = -1*i_p;								% of the sample
		end;
		
		i_p = 2*i_p;									% Double the sample value since the absolute
														% value ranges only from 0 to +127

		% Low pass filter the signal power
		% Transfer function is beta*P(n-1) + (1-beta)*abs(current sample)
		% beta = 0.95
		tmp1 = mymul(Spower_mu1,spower);				% beta*previous signal power value		
		tmp2 = mymul(Spower_mu2,i_p);					% (1-beta)*abs(current sample)
		spower = tmp1 + tmp2;							% Calculate current signal power
		
		if(spower > Noise_floor)						% If signal power > noise floor
			spower_counter = spower_counter - 1;		% decrement signal power counter
			if(Scounts > 80 | spower_counter < 10)		% If signal power is greater than noise
														% floor for more than 20 ms...
				if(spower < min_spower)					% If current signal power is less than the
					min_spower = spower;				% minimum signal power, update minimum
				end;									% signal power
				max_spower = 0;							% Reset maximum signal power to 0
			end;
			DT = 1;										% Signal is detected
		
		else											% If signal power < noise floor
			npower_counter = npower_counter - 1;		% decrement noise power counter
			if(Ncounts > 25 | npower_counter < 5)		% If signal power is less than noise
														% floor for more than 5 ms...
				if(spower > max_spower)					% If current signal power is greater than 	
					max_spower = spower;				% the maximum signal power, update the
				end;									% maximum signal power
				min_spower = 256;						% Reset minimum signal power to 256
			end;
			DT = 0;										% Signal not present
		end;

		if(spower_counter==0);							% If signal power counter is zero,
														% exponentially increase noise floor
			tmp1 = (Noise_floor + floor(Noise_floor/2)); % tmp1 = 1.5*Noise_floor
			if(tmp1 <= floor(min_spower/2))				% if tmp1 is less than or equal to the
				Noise_floor = tmp1;						% minimum signal power, update the 
			end;										% noise floor
			if(Scounts <= 320)							% If Scounts is less than or equal to 320
				Scounts = 2*Scounts;					% update Scounts
			end;										
			spower_counter = Scounts;					% Reset signal power counter
			Ncounts = 25;								% Reset Ncounts
			npower_counter = 25;						% Reset noise power counter
			
		elseif(npower_counter==0)						% If noise power counter is zero,
														% exponentially decrease noise floor
			tmp1 = floor(Noise_floor/2);				% tmp1 = Noise_floor/2
			if((tmp1 >= (max_spower + floor(max_spower/2))) & (tmp1 > 0)) % If tmp1 > 0 and
														% greater than or equal to 
														% 1.5*(maximum signal power)
				Noise_floor = tmp1;						% update noise floor
			end;
			if(Ncounts <= 50)							% If Ncounts is less than or equal to
				Ncounts = 2*Ncounts;					% 50, update Ncounts
			end;
			npower_counter = Ncounts;					% Reset noise power counter
			Scounts = 80;								% Reset Scounts
			spower_counter = 80;						% Reset signal power counter
		end;
	
%{------------------ DECISION LOGIC ----------------------------}

		% This decision logic is implemented at a further decimated rate
		% of 2 kHz

		% Notation used in decision logic
		% DT = Detect signal				CT = Count
		% PS = Previous symbol				ST = Signal timer
		% TC = Timer count					NS = New symbol
		% VS = Valid symbol

		if (rem(n,4) == 0)									% Execute every fourth sample
			if((NS <= 16) & (DT == 1))						% If signal is detected and the
				TC = N10ms;									% value of DTMF symbol is less than or
															% equal to 16, reset TC
															
				if(PS == NS)								% If new symbol is same as last one
					if(ST > N5ms)							% and the signal is stable for more than
						if(VS == NS)						% 10 ms and the valid symbol and current
							CT = CT + ST;					% symbol are same, update signal duration
						else								% with ST. Else reset signal
							CT = ST;						% duration to the value of ST
						end;
						VS = NS;							% Store current symbol as valid symbol
						ST = 0;								% Reset ST
					else
						ST = ST + 1;						% Increment ST by 1
					end;	
				else										% If current symbol and previous symbol are
					if((ST > 0) & (VS == PS)) 				% not same, but if previous
															% symbol and valid symbol are same, and 
															% ST > 0, then there was a pause
						CT = CT + ST;						% Update signal duration
					end;
					ST = 0;									% Reset ST
				end;
				PS = NS;									% Store current symbol as previous symbol
			
			else											% If signal is not present or if DTMF symbol
															% is not present
				if((ST > 0) & (VS == PS))
					CT = CT + ST;							% Update signal duration
				end;		
				ST = 0;										% Reset ST
				if(TC == 0)									% If signal is not present for more than
					if(CT > N23ms)							% 10 ms, check if the DTMF symbol duration 	
						key(index_count) = VS;				% meets ITU duration constraint. If symbol 
						index_count = index_count + 1;		% duration > 23 ms, store valid symbol in
					end;									% output array
					CT = 0;									% Reset signal duration
					VS = 19;								% Reset valid symbol
					PS = 20;								% Reset previous symbol
				else		
					if(DT == 0)								% If signal is not present, decrease
						TC = TC - 1;						% TC by 1
					end;
				end;
			end;
		end;		
			
%{--------------- END PROGRAM -----------------------------------}
	end; %end if of decimation

end;%end main for loop	
